"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const parser_1 = require("./parser");
const cssSchema = __importStar(require("./css-schema"));
const built_in_1 = __importDefault(require("./built-in"));
const startsWith_1 = __importDefault(require("lodash/startsWith"));
const find_1 = __importDefault(require("lodash/find"));
const compact_1 = __importDefault(require("lodash/compact"));
const zip_1 = __importDefault(require("lodash/zip"));
function prepareName(name) {
    return name.replace(/\{|\}/g, '').trim();
}
function isClassOrId(currentWord) {
    return /^[.#&]/.test(currentWord);
}
exports.isClassOrId = isClassOrId;
function isAtRule(currentWord) {
    return startsWith_1.default(currentWord, '@');
}
exports.isAtRule = isAtRule;
function isValue(cssSchema, currentWord) {
    const property = getPropertyName(currentWord);
    return !!property && Boolean(findPropertySchema(cssSchema, property));
}
exports.isValue = isValue;
function getPropertyName(currentWord) {
    return currentWord
        .trim()
        .replace(':', ' ')
        .split(' ')[0];
}
exports.getPropertyName = getPropertyName;
function findPropertySchema(cssSchema, property) {
    const properties = cssSchema.data.css.properties;
    return find_1.default(properties, item => item.name === property);
}
exports.findPropertySchema = findPropertySchema;
function _variableSymbol(node, text, currentWord) {
    const name = node.name;
    const lineno = Number(node.val.lineno) - 1;
    const completionItem = vscode_languageserver_types_1.CompletionItem.create(name);
    completionItem.detail = text[lineno].trim();
    completionItem.kind = vscode_languageserver_types_1.CompletionItemKind.Variable;
    return completionItem;
}
function _functionSymbol(node, text) {
    const name = node.name;
    const completionItem = vscode_languageserver_types_1.CompletionItem.create(name);
    completionItem.kind = vscode_languageserver_types_1.CompletionItemKind.Function;
    return completionItem;
}
function _selectorSymbol(node, text, currentWord) {
    const firstSegment = node.segments[0];
    const name = firstSegment.string
        ? node.segments.map(s => s.string).join('')
        : firstSegment.nodes.map(s => s.name).join('');
    const completionItem = vscode_languageserver_types_1.CompletionItem.create(name);
    completionItem.kind = vscode_languageserver_types_1.CompletionItemKind.Class;
    return completionItem;
}
function _selectorCallSymbol(node, text) {
    const lineno = Number(node.lineno) - 1;
    const name = prepareName(text[lineno]);
    const completionItem = vscode_languageserver_types_1.CompletionItem.create(name);
    completionItem.kind = vscode_languageserver_types_1.CompletionItemKind.Class;
    return completionItem;
}
function isVisible(useSite, defSite) {
    if (!useSite || !defSite) {
        return true;
    }
    if (useSite.length < defSite.length) {
        return false;
    }
    for (const [use, def] of zip_1.default(useSite, defSite)) {
        if (use && def && use > def) {
            return false;
        }
    }
    return true;
}
function getAllSymbols(text, currentWord, position) {
    const ast = parser_1.buildAst(text);
    if (!ast) {
        return [];
    }
    const node = parser_1.findNodeAtPosition(ast, position);
    const scope = node ? node.__scope : undefined;
    const splittedText = text.split('\n');
    const rawSymbols = parser_1.flattenAndFilterAst(ast).filter(item => ['Media', 'Keyframes', 'Atrule', 'Import', 'Require', 'Supports', 'Literal'].indexOf(item.__type) === -1);
    return compact_1.default(rawSymbols.map(item => {
        if (!isVisible(scope, item.__scope)) {
            return undefined;
        }
        if (parser_1.isVariableNode(item)) {
            return _variableSymbol(item, splittedText, currentWord);
        }
        if (parser_1.isFunctionNode(item)) {
            return _functionSymbol(item, splittedText);
        }
        if (parser_1.isSelectorNode(item)) {
            return _selectorSymbol(item, splittedText, currentWord);
        }
        if (parser_1.isSelectorCallNode(item)) {
            return _selectorCallSymbol(item, splittedText);
        }
    }));
}
exports.getAllSymbols = getAllSymbols;
function getAtRules(cssSchema, currentWord) {
    if (!isAtRule(currentWord)) {
        return [];
    }
    return cssSchema.data.css.atdirectives.map(property => {
        const completionItem = vscode_languageserver_types_1.CompletionItem.create(property.name);
        completionItem.detail = property.desc;
        completionItem.kind = vscode_languageserver_types_1.CompletionItemKind.Keyword;
        return completionItem;
    });
}
exports.getAtRules = getAtRules;
function getProperties(cssSchema, currentWord, useSeparator) {
    if (isClassOrId(currentWord) || isAtRule(currentWord)) {
        return [];
    }
    return cssSchema.data.css.properties.map(property => {
        const completionItem = vscode_languageserver_types_1.CompletionItem.create(property.name);
        completionItem.insertText = property.name + (useSeparator ? ': ' : ' ');
        completionItem.detail = property.desc;
        completionItem.kind = vscode_languageserver_types_1.CompletionItemKind.Property;
        return completionItem;
    });
}
exports.getProperties = getProperties;
function getValues(cssSchema, currentWord) {
    const property = getPropertyName(currentWord);
    const result = findPropertySchema(cssSchema, property);
    const values = result && result.values;
    if (!values) {
        return [];
    }
    return values.map(property => {
        const completionItem = vscode_languageserver_types_1.CompletionItem.create(property.name);
        completionItem.documentation = property.desc;
        completionItem.kind = vscode_languageserver_types_1.CompletionItemKind.Value;
        return completionItem;
    });
}
exports.getValues = getValues;
function provideCompletionItems(document, position) {
    const start = document.offsetAt(vscode_languageserver_types_1.Position.create(position.line, 0));
    const end = document.offsetAt(position);
    const text = document.getText();
    const currentWord = text.slice(start, end).trim();
    const value = isValue(cssSchema, currentWord);
    let completions = [];
    if (value) {
        const values = getValues(cssSchema, currentWord);
        const symbols = getAllSymbols(text, currentWord, position).filter(item => item.kind === vscode_languageserver_types_1.CompletionItemKind.Variable || item.kind === vscode_languageserver_types_1.CompletionItemKind.Function);
        completions = completions.concat(values, symbols, built_in_1.default);
    }
    else {
        const atRules = getAtRules(cssSchema, currentWord);
        const properties = getProperties(cssSchema, currentWord, false);
        const symbols = getAllSymbols(text, currentWord, position).filter(item => item.kind !== vscode_languageserver_types_1.CompletionItemKind.Variable);
        completions = completions.concat(properties, atRules, symbols);
    }
    return {
        isIncomplete: false,
        items: completions
    };
}
exports.provideCompletionItems = provideCompletionItems;
//# sourceMappingURL=completion-item.js.map