"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const parser_1 = require("./parser");
const uniqBy_1 = __importDefault(require("lodash/uniqBy"));
const compact_1 = __importDefault(require("lodash/compact"));
function _buildHashFromSymbol(symbol) {
    return `${symbol.kind}_${symbol.name}_${symbol.location.range.start.line}_${symbol.location.range.end.line}`;
}
function prepareName(name) {
    return name.replace(/\{|\}/g, '').trim();
}
function _variableSymbol(node, text) {
    const name = node.name;
    const lineno = Number(node.val.lineno) - 1;
    const column = Math.max(text[lineno].indexOf(name), 0);
    const range = vscode_languageserver_types_1.Range.create(lineno, column, lineno, column + name.length);
    return vscode_languageserver_types_1.SymbolInformation.create(name, vscode_languageserver_types_1.SymbolKind.Variable, range);
}
function _functionSymbol(node, text) {
    const name = node.name;
    const lineno = Number(node.val.lineno) - 1;
    const column = Math.max(text[lineno].indexOf(name), 0);
    const posStart = vscode_languageserver_types_1.Position.create(lineno, column);
    const posEnd = vscode_languageserver_types_1.Position.create(lineno, column + name.length);
    const range = vscode_languageserver_types_1.Range.create(posStart, posEnd);
    return vscode_languageserver_types_1.SymbolInformation.create(name, vscode_languageserver_types_1.SymbolKind.Function, range);
}
function _selectorSymbol(node, text) {
    const firstSegment = node.segments[0];
    const name = firstSegment.string
        ? node.segments.map(s => s.string).join('')
        : firstSegment.nodes.map(s => s.name).join('');
    const lineno = Number(firstSegment.lineno) - 1;
    const column = node.column - 1;
    const posStart = vscode_languageserver_types_1.Position.create(lineno, column);
    const posEnd = vscode_languageserver_types_1.Position.create(lineno, column + name.length);
    const range = vscode_languageserver_types_1.Range.create(posStart, posEnd);
    return vscode_languageserver_types_1.SymbolInformation.create(name, vscode_languageserver_types_1.SymbolKind.Class, range);
}
function _selectorCallSymbol(node, text) {
    const lineno = Number(node.lineno) - 1;
    const name = prepareName(text[lineno]);
    const column = Math.max(text[lineno].indexOf(name), 0);
    const posStart = vscode_languageserver_types_1.Position.create(lineno, column);
    const posEnd = vscode_languageserver_types_1.Position.create(lineno, column + name.length);
    return vscode_languageserver_types_1.SymbolInformation.create(name, vscode_languageserver_types_1.SymbolKind.Class, vscode_languageserver_types_1.Range.create(posStart, posEnd));
}
function _atRuleSymbol(node, text) {
    const lineno = Number(node.lineno) - 1;
    const name = prepareName(text[lineno]);
    const column = Math.max(text[lineno].indexOf(name), 0);
    const posStart = vscode_languageserver_types_1.Position.create(lineno, column);
    const posEnd = vscode_languageserver_types_1.Position.create(lineno, column + name.length);
    return vscode_languageserver_types_1.SymbolInformation.create(name, vscode_languageserver_types_1.SymbolKind.Namespace, vscode_languageserver_types_1.Range.create(posStart, posEnd));
}
function processRawSymbols(rawSymbols, text) {
    return compact_1.default(rawSymbols.map(symNode => {
        if (parser_1.isVariableNode(symNode)) {
            return _variableSymbol(symNode, text);
        }
        if (parser_1.isFunctionNode(symNode)) {
            return _functionSymbol(symNode, text);
        }
        if (parser_1.isSelectorNode(symNode)) {
            return _selectorSymbol(symNode, text);
        }
        if (parser_1.isSelectorCallNode(symNode)) {
            return _selectorCallSymbol(symNode, text);
        }
        if (parser_1.isAtRuleNode(symNode)) {
            return _atRuleSymbol(symNode, text);
        }
    }));
}
function provideDocumentSymbols(document) {
    const text = document.getText();
    const ast = parser_1.buildAst(text);
    if (!ast) {
        return [];
    }
    const rawSymbols = compact_1.default(parser_1.flattenAndFilterAst(ast));
    const symbolInfos = processRawSymbols(rawSymbols, text.split('\n'));
    return uniqBy_1.default(symbolInfos, _buildHashFromSymbol);
}
exports.provideDocumentSymbols = provideDocumentSymbols;
//# sourceMappingURL=symbols-finder.js.map